@@ -13,10 +13,11 @@ pub trait NativeDialogExtManual {
     /// `ResponseType` on response.
     ///
     /// ```no_run
+    /// # use gtk4 as gtk;
     /// use gtk::prelude::*;
     ///
     /// # async fn run() {
-    /// let dialog = gtk::FileChooserNative::builder()
+    /// let dialog = gtk::FileChooserNativeBuilder::new()
     ///    .title("Select a File")
     ///    .build();
     ///
@@ -44,9 +45,7 @@ impl<O: IsA<NativeDialog>> NativeDialogExtManual for O {
             self.show();
 
             if let Ok(response) = receiver.await {
-                if response != ResponseType::DeleteEvent {
-                    self.disconnect(response_handler);
-                }
+                self.disconnect(response_handler);
                 response
             } else {
                 ResponseType::None
