@@ -3,6 +3,7 @@
 use glib::translate::*;
 use libc::c_char;
 
+#[doc(alias = "GtkRecentData")]
 pub struct RecentData {
     pub display_name: Option<String>,
     pub description: Option<String>,
@@ -15,6 +16,7 @@ pub struct RecentData {
 
 #[doc(hidden)]
 impl<'a> ToGlibPtr<'a, *mut ffi::GtkRecentData> for RecentData {
+    #[allow(clippy::type_complexity)]
     type Storage = (
         Box<ffi::GtkRecentData>,
         [Stash<'a, *mut c_char, Option<String>>; 2],
