@@ -1,15 +1,12 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use glib::subclass::prelude::*;
+use crate::subclass::prelude::*;
+use crate::{ApplicationWindow, Window};
 
-use super::window::WindowImpl;
-use crate::ApplicationWindow;
-use crate::Window;
-
-pub trait ApplicationWindowImpl: WindowImpl {}
+pub trait ApplicationWindowImpl: WindowImpl + 'static {}
 
 unsafe impl<T: ApplicationWindowImpl> IsSubclassable<T> for ApplicationWindow {
-    fn class_init(class: &mut ::glib::Class<Self>) {
+    fn class_init(class: &mut glib::Class<Self>) {
         <Window as IsSubclassable<T>>::class_init(class);
     }
 
