@@ -1,44 +1,42 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use glib::subclass::prelude::*;
+use crate::subclass::prelude::*;
+use crate::{Button, Widget};
 use glib::translate::*;
 use glib::Cast;
 
-use super::bin::BinImpl;
-use crate::Button;
-
-pub trait ButtonImpl: ButtonImplExt + BinImpl {
-    fn activate(&self) {
-        self.parent_activate()
+pub trait ButtonImpl: ButtonImplExt + WidgetImpl {
+    fn activate(&self, button: &Self::Type) {
+        self.parent_activate(button)
     }
 
-    fn clicked(&self) {
-        self.parent_clicked()
+    fn clicked(&self, button: &Self::Type) {
+        self.parent_clicked(button)
     }
 }
 
 pub trait ButtonImplExt: ObjectSubclass {
-    fn parent_activate(&self);
-    fn parent_clicked(&self);
+    fn parent_activate(&self, button: &Self::Type);
+    fn parent_clicked(&self, button: &Self::Type);
 }
 
 impl<T: ButtonImpl> ButtonImplExt for T {
-    fn parent_activate(&self) {
+    fn parent_activate(&self, button: &Self::Type) {
         unsafe {
             let data = T::type_data();
             let parent_class = data.as_ref().parent_class() as *mut ffi::GtkButtonClass;
             if let Some(f) = (*parent_class).activate {
-                f(self.obj().unsafe_cast_ref::<Button>().to_glib_none().0)
+                f(button.unsafe_cast_ref::<Button>().to_glib_none().0)
             }
         }
     }
 
-    fn parent_clicked(&self) {
+    fn parent_clicked(&self, button: &Self::Type) {
         unsafe {
             let data = T::type_data();
             let parent_class = data.as_ref().parent_class() as *mut ffi::GtkButtonClass;
             if let Some(f) = (*parent_class).clicked {
-                f(self.obj().unsafe_cast_ref::<Button>().to_glib_none().0)
+                f(button.unsafe_cast_ref::<Button>().to_glib_none().0)
             }
         }
     }
@@ -46,28 +44,30 @@ impl<T: ButtonImpl> ButtonImplExt for T {
 
 unsafe impl<T: ButtonImpl> IsSubclassable<T> for Button {
     fn class_init(class: &mut glib::Class<Self>) {
-        Self::parent_class_init::<T>(class);
-
-        if !crate::rt::is_initialized() {
-            panic!("GTK has to be initialized first");
-        }
+        <Widget as IsSubclassable<T>>::class_init(class);
 
         let klass = class.as_mut();
         klass.activate = Some(button_activate::<T>);
         klass.clicked = Some(button_clicked::<T>);
     }
+
+    fn instance_init(instance: &mut glib::subclass::InitializingObject<T>) {
+        <Widget as IsSubclassable<T>>::instance_init(instance);
+    }
 }
 
 unsafe extern "C" fn button_activate<T: ButtonImpl>(ptr: *mut ffi::GtkButton) {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
+    let imp = instance.impl_();
+    let wrap: Borrowed<Button> = from_glib_borrow(ptr);
 
-    imp.activate()
+    imp.activate(wrap.unsafe_cast_ref())
 }
 
 unsafe extern "C" fn button_clicked<T: ButtonImpl>(ptr: *mut ffi::GtkButton) {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
+    let imp = instance.impl_();
+    let wrap: Borrowed<Button> = from_glib_borrow(ptr);
 
-    imp.clicked()
+    imp.clicked(wrap.unsafe_cast_ref())
 }
