@@ -3,65 +3,16 @@
 use libc::{c_char, c_int};
 use std::mem;
 
+use crate::subclass::prelude::*;
+use crate::{CellEditable, CellRenderer, CellRendererState, SizeRequestMode, Snapshot, Widget};
 use glib::object::IsA;
-use glib::subclass::prelude::*;
 use glib::translate::*;
-use glib::Cast;
-use glib::GString;
-
-use crate::CellEditable;
-use crate::CellRenderer;
-use crate::CellRendererState;
-use crate::SizeRequestMode;
-use crate::Widget;
+use glib::{Cast, GString, Object};
 
 pub trait CellRendererImpl: CellRendererImplExt + ObjectImpl {
-    fn request_mode(&self) -> SizeRequestMode {
-        self.parent_request_mode()
-    }
-
-    fn preferred_width<P: IsA<Widget>>(&self, widget: &P) -> (i32, i32) {
-        self.parent_preferred_width(widget)
-    }
-
-    #[doc(alias = "get_preferred_width_for_height")]
-    fn preferred_width_for_height<P: IsA<Widget>>(&self, widget: &P, height: i32) -> (i32, i32) {
-        self.parent_preferred_width_for_height(widget, height)
-    }
-
-    #[doc(alias = "get_preferred_height")]
-    fn preferred_height<P: IsA<Widget>>(&self, widget: &P) -> (i32, i32) {
-        self.parent_preferred_height(widget)
-    }
-
-    #[doc(alias = "get_preferred_height_for_width")]
-    fn preferred_height_for_width<P: IsA<Widget>>(&self, widget: &P, width: i32) -> (i32, i32) {
-        self.parent_preferred_height_for_width(widget, width)
-    }
-
-    #[doc(alias = "get_aligned_area")]
-    fn aligned_area<P: IsA<Widget>>(
-        &self,
-        widget: &P,
-        flags: CellRendererState,
-        cell_area: &gdk::Rectangle,
-    ) -> gdk::Rectangle {
-        self.parent_aligned_area(widget, flags, cell_area)
-    }
-
-    fn render<P: IsA<Widget>>(
-        &self,
-        cr: &cairo::Context,
-        widget: &P,
-        background_area: &gdk::Rectangle,
-        cell_area: &gdk::Rectangle,
-        flags: CellRendererState,
-    ) {
-        self.parent_render(cr, widget, background_area, cell_area, flags);
-    }
-
     fn activate<P: IsA<Widget>>(
         &self,
+        renderer: &Self::Type,
         event: Option<&gdk::Event>,
         widget: &P,
         path: &str,
@@ -69,11 +20,93 @@ pub trait CellRendererImpl: CellRendererImplExt + ObjectImpl {
         cell_area: &gdk::Rectangle,
         flags: CellRendererState,
     ) -> bool {
-        self.parent_activate(event, widget, path, background_area, cell_area, flags)
+        self.parent_activate(
+            renderer,
+            event,
+            widget,
+            path,
+            background_area,
+            cell_area,
+            flags,
+        )
+    }
+
+    fn editing_canceled(&self, renderer: &Self::Type) {
+        self.parent_editing_canceled(renderer)
+    }
+
+    fn editing_started(&self, renderer: &Self::Type, editable: &CellEditable, path: &str) {
+        self.parent_editing_started(renderer, editable, path)
+    }
+
+    #[doc(alias = "get_aligned_area")]
+    fn aligned_area<P: IsA<Widget>>(
+        &self,
+        renderer: &Self::Type,
+        widget: &P,
+        flags: CellRendererState,
+        cell_area: &gdk::Rectangle,
+    ) -> gdk::Rectangle {
+        self.parent_aligned_area(renderer, widget, flags, cell_area)
+    }
+
+    #[doc(alias = "get_preferred_height_for_width")]
+    fn preferred_height_for_width<P: IsA<Widget>>(
+        &self,
+        renderer: &Self::Type,
+        widget: &P,
+        width: i32,
+    ) -> (i32, i32) {
+        self.parent_preferred_height_for_width(renderer, widget, width)
+    }
+
+    #[doc(alias = "get_preferred_height")]
+    fn preferred_height<P: IsA<Widget>>(&self, renderer: &Self::Type, widget: &P) -> (i32, i32) {
+        self.parent_preferred_height(renderer, widget)
+    }
+
+    #[doc(alias = "get_preferred_width_for_height")]
+    fn preferred_width_for_height<P: IsA<Widget>>(
+        &self,
+        renderer: &Self::Type,
+        widget: &P,
+        height: i32,
+    ) -> (i32, i32) {
+        self.parent_preferred_width_for_height(renderer, widget, height)
+    }
+
+    #[doc(alias = "get_preferred_width")]
+    fn preferred_width<P: IsA<Widget>>(&self, renderer: &Self::Type, widget: &P) -> (i32, i32) {
+        self.parent_preferred_width(renderer, widget)
+    }
+
+    #[doc(alias = "get_request_mode")]
+    fn request_mode(&self, renderer: &Self::Type) -> SizeRequestMode {
+        self.parent_request_mode(renderer)
+    }
+
+    fn snapshot<P: IsA<Widget>>(
+        &self,
+        renderer: &Self::Type,
+        snapshot: &Snapshot,
+        widget: &P,
+        background_area: &gdk::Rectangle,
+        cell_area: &gdk::Rectangle,
+        flags: CellRendererState,
+    ) {
+        self.parent_snapshot(
+            renderer,
+            snapshot,
+            widget,
+            background_area,
+            cell_area,
+            flags,
+        );
     }
 
     fn start_editing<P: IsA<Widget>>(
         &self,
+        renderer: &Self::Type,
         event: Option<&gdk::Event>,
         widget: &P,
         path: &str,
@@ -81,48 +114,22 @@ pub trait CellRendererImpl: CellRendererImplExt + ObjectImpl {
         cell_area: &gdk::Rectangle,
         flags: CellRendererState,
     ) -> Option<CellEditable> {
-        self.parent_start_editing(event, widget, path, background_area, cell_area, flags)
-    }
-
-    fn editing_canceled(&self) {
-        self.parent_editing_canceled()
-    }
-
-    fn editing_started(&self, editable: &CellEditable, path: &str) {
-        self.parent_editing_started(editable, path)
+        self.parent_start_editing(
+            renderer,
+            event,
+            widget,
+            path,
+            background_area,
+            cell_area,
+            flags,
+        )
     }
 }
 
 pub trait CellRendererImplExt: ObjectSubclass {
-    fn parent_request_mode(&self) -> SizeRequestMode;
-    fn parent_preferred_width<P: IsA<Widget>>(&self, widget: &P) -> (i32, i32);
-    fn parent_preferred_width_for_height<P: IsA<Widget>>(
-        &self,
-        widget: &P,
-        height: i32,
-    ) -> (i32, i32);
-    fn parent_preferred_height<P: IsA<Widget>>(&self, widget: &P) -> (i32, i32);
-    fn parent_preferred_height_for_width<P: IsA<Widget>>(
-        &self,
-        widget: &P,
-        width: i32,
-    ) -> (i32, i32);
-    fn parent_aligned_area<P: IsA<Widget>>(
-        &self,
-        widget: &P,
-        flags: CellRendererState,
-        cell_area: &gdk::Rectangle,
-    ) -> gdk::Rectangle;
-    fn parent_render<P: IsA<Widget>>(
-        &self,
-        cr: &cairo::Context,
-        widget: &P,
-        background_area: &gdk::Rectangle,
-        cell_area: &gdk::Rectangle,
-        flags: CellRendererState,
-    );
     fn parent_activate<P: IsA<Widget>>(
         &self,
+        renderer: &Self::Type,
         event: Option<&gdk::Event>,
         widget: &P,
         path: &str,
@@ -130,8 +137,50 @@ pub trait CellRendererImplExt: ObjectSubclass {
         cell_area: &gdk::Rectangle,
         flags: CellRendererState,
     ) -> bool;
+    fn parent_editing_canceled(&self, renderer: &Self::Type);
+    fn parent_editing_started(&self, renderer: &Self::Type, editable: &CellEditable, path: &str);
+    fn parent_aligned_area<P: IsA<Widget>>(
+        &self,
+        renderer: &Self::Type,
+        widget: &P,
+        flags: CellRendererState,
+        cell_area: &gdk::Rectangle,
+    ) -> gdk::Rectangle;
+    fn parent_preferred_height_for_width<P: IsA<Widget>>(
+        &self,
+        renderer: &Self::Type,
+        widget: &P,
+        width: i32,
+    ) -> (i32, i32);
+    fn parent_preferred_height<P: IsA<Widget>>(
+        &self,
+        renderer: &Self::Type,
+        widget: &P,
+    ) -> (i32, i32);
+    fn parent_preferred_width_for_height<P: IsA<Widget>>(
+        &self,
+        renderer: &Self::Type,
+        widget: &P,
+        height: i32,
+    ) -> (i32, i32);
+    fn parent_preferred_width<P: IsA<Widget>>(
+        &self,
+        renderer: &Self::Type,
+        widget: &P,
+    ) -> (i32, i32);
+    fn parent_request_mode(&self, renderer: &Self::Type) -> SizeRequestMode;
+    fn parent_snapshot<P: IsA<Widget>>(
+        &self,
+        renderer: &Self::Type,
+        snapshot: &Snapshot,
+        widget: &P,
+        background_area: &gdk::Rectangle,
+        cell_area: &gdk::Rectangle,
+        flags: CellRendererState,
+    );
     fn parent_start_editing<P: IsA<Widget>>(
         &self,
+        renderer: &Self::Type,
         event: Option<&gdk::Event>,
         widget: &P,
         path: &str,
@@ -139,25 +188,26 @@ pub trait CellRendererImplExt: ObjectSubclass {
         cell_area: &gdk::Rectangle,
         flags: CellRendererState,
     ) -> Option<CellEditable>;
-    fn parent_editing_canceled(&self);
-    fn parent_editing_started(&self, editable: &CellEditable, path: &str);
 }
 
 impl<T: CellRendererImpl> CellRendererImplExt for T {
-    fn parent_request_mode(&self) -> SizeRequestMode {
+    fn parent_request_mode(&self, renderer: &Self::Type) -> SizeRequestMode {
         unsafe {
             let data = T::type_data();
             let parent_class = data.as_ref().parent_class() as *mut ffi::GtkCellRendererClass;
             let f = (*parent_class).get_request_mode.unwrap();
-            from_glib(f(self
-                .obj()
+            from_glib(f(renderer
                 .unsafe_cast_ref::<CellRenderer>()
                 .to_glib_none()
                 .0))
         }
     }
 
-    fn parent_preferred_width<P: IsA<Widget>>(&self, widget: &P) -> (i32, i32) {
+    fn parent_preferred_width<P: IsA<Widget>>(
+        &self,
+        renderer: &Self::Type,
+        widget: &P,
+    ) -> (i32, i32) {
         unsafe {
             let data = T::type_data();
             let parent_class = data.as_ref().parent_class() as *mut ffi::GtkCellRendererClass;
@@ -166,10 +216,7 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
             let mut minimum_size = mem::MaybeUninit::uninit();
             let mut natural_size = mem::MaybeUninit::uninit();
             f(
-                self.obj()
-                    .unsafe_cast_ref::<CellRenderer>()
-                    .to_glib_none()
-                    .0,
+                renderer.unsafe_cast_ref::<CellRenderer>().to_glib_none().0,
                 widget.as_ref().to_glib_none().0,
                 minimum_size.as_mut_ptr(),
                 natural_size.as_mut_ptr(),
@@ -180,6 +227,7 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
 
     fn parent_preferred_width_for_height<P: IsA<Widget>>(
         &self,
+        renderer: &Self::Type,
         widget: &P,
         height: i32,
     ) -> (i32, i32) {
@@ -191,10 +239,7 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
             let mut minimum_size = mem::MaybeUninit::uninit();
             let mut natural_size = mem::MaybeUninit::uninit();
             f(
-                self.obj()
-                    .unsafe_cast_ref::<CellRenderer>()
-                    .to_glib_none()
-                    .0,
+                renderer.unsafe_cast_ref::<CellRenderer>().to_glib_none().0,
                 widget.as_ref().to_glib_none().0,
                 height,
                 minimum_size.as_mut_ptr(),
@@ -203,7 +248,11 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
             (minimum_size.assume_init(), natural_size.assume_init())
         }
     }
-    fn parent_preferred_height<P: IsA<Widget>>(&self, widget: &P) -> (i32, i32) {
+    fn parent_preferred_height<P: IsA<Widget>>(
+        &self,
+        renderer: &Self::Type,
+        widget: &P,
+    ) -> (i32, i32) {
         unsafe {
             let data = T::type_data();
             let parent_class = data.as_ref().parent_class() as *mut ffi::GtkCellRendererClass;
@@ -211,10 +260,7 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
             let mut minimum_size = mem::MaybeUninit::uninit();
             let mut natural_size = mem::MaybeUninit::uninit();
             f(
-                self.obj()
-                    .unsafe_cast_ref::<CellRenderer>()
-                    .to_glib_none()
-                    .0,
+                renderer.unsafe_cast_ref::<CellRenderer>().to_glib_none().0,
                 widget.as_ref().to_glib_none().0,
                 minimum_size.as_mut_ptr(),
                 natural_size.as_mut_ptr(),
@@ -224,6 +270,7 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
     }
     fn parent_preferred_height_for_width<P: IsA<Widget>>(
         &self,
+        renderer: &Self::Type,
         widget: &P,
         width: i32,
     ) -> (i32, i32) {
@@ -234,10 +281,7 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
             let mut minimum_size = mem::MaybeUninit::uninit();
             let mut natural_size = mem::MaybeUninit::uninit();
             f(
-                self.obj()
-                    .unsafe_cast_ref::<CellRenderer>()
-                    .to_glib_none()
-                    .0,
+                renderer.unsafe_cast_ref::<CellRenderer>().to_glib_none().0,
                 widget.as_ref().to_glib_none().0,
                 width,
                 minimum_size.as_mut_ptr(),
@@ -249,6 +293,7 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
 
     fn parent_aligned_area<P: IsA<Widget>>(
         &self,
+        renderer: &Self::Type,
         widget: &P,
         flags: CellRendererState,
         cell_area: &gdk::Rectangle,
@@ -259,10 +304,7 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
             let mut aligned_area = gdk::Rectangle::uninitialized();
             let f = (*parent_class).get_aligned_area.unwrap();
             f(
-                self.obj()
-                    .unsafe_cast_ref::<CellRenderer>()
-                    .to_glib_none()
-                    .0,
+                renderer.unsafe_cast_ref::<CellRenderer>().to_glib_none().0,
                 widget.as_ref().to_glib_none().0,
                 flags.into_glib(),
                 cell_area.to_glib_none().0,
@@ -272,9 +314,10 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
         }
     }
 
-    fn parent_render<P: IsA<Widget>>(
+    fn parent_snapshot<P: IsA<Widget>>(
         &self,
-        cr: &cairo::Context,
+        renderer: &Self::Type,
+        snapshot: &Snapshot,
         widget: &P,
         background_area: &gdk::Rectangle,
         cell_area: &gdk::Rectangle,
@@ -283,13 +326,10 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
         unsafe {
             let data = T::type_data();
             let parent_class = data.as_ref().parent_class() as *mut ffi::GtkCellRendererClass;
-            if let Some(f) = (*parent_class).render {
+            if let Some(f) = (*parent_class).snapshot {
                 f(
-                    self.obj()
-                        .unsafe_cast_ref::<CellRenderer>()
-                        .to_glib_none()
-                        .0,
-                    cr.to_glib_none().0,
+                    renderer.unsafe_cast_ref::<CellRenderer>().to_glib_none().0,
+                    snapshot.to_glib_none().0,
                     widget.as_ref().to_glib_none().0,
                     background_area.to_glib_none().0,
                     cell_area.to_glib_none().0,
@@ -301,6 +341,7 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
 
     fn parent_activate<P: IsA<Widget>>(
         &self,
+        renderer: &Self::Type,
         event: Option<&gdk::Event>,
         widget: &P,
         path: &str,
@@ -313,10 +354,7 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
             let parent_class = data.as_ref().parent_class() as *mut ffi::GtkCellRendererClass;
             if let Some(f) = (*parent_class).activate {
                 from_glib(f(
-                    self.obj()
-                        .unsafe_cast_ref::<CellRenderer>()
-                        .to_glib_none()
-                        .0,
+                    renderer.unsafe_cast_ref::<CellRenderer>().to_glib_none().0,
                     mut_override(event.to_glib_none().0),
                     widget.as_ref().to_glib_none().0,
                     path.to_glib_none().0,
@@ -332,6 +370,7 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
 
     fn parent_start_editing<P: IsA<Widget>>(
         &self,
+        renderer: &Self::Type,
         event: Option<&gdk::Event>,
         widget: &P,
         path: &str,
@@ -344,10 +383,7 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
             let parent_class = data.as_ref().parent_class() as *mut ffi::GtkCellRendererClass;
             if let Some(f) = (*parent_class).start_editing {
                 from_glib_none(f(
-                    self.obj()
-                        .unsafe_cast_ref::<CellRenderer>()
-                        .to_glib_none()
-                        .0,
+                    renderer.unsafe_cast_ref::<CellRenderer>().to_glib_none().0,
                     mut_override(event.to_glib_none().0),
                     widget.as_ref().to_glib_none().0,
                     path.to_glib_none().0,
@@ -361,30 +397,23 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
         }
     }
 
-    fn parent_editing_canceled(&self) {
+    fn parent_editing_canceled(&self, renderer: &Self::Type) {
         unsafe {
             let data = T::type_data();
             let parent_class = data.as_ref().parent_class() as *mut ffi::GtkCellRendererClass;
             if let Some(f) = (*parent_class).editing_canceled {
-                f(self
-                    .obj()
-                    .unsafe_cast_ref::<CellRenderer>()
-                    .to_glib_none()
-                    .0)
+                f(renderer.unsafe_cast_ref::<CellRenderer>().to_glib_none().0)
             }
         }
     }
 
-    fn parent_editing_started(&self, editable: &CellEditable, path: &str) {
+    fn parent_editing_started(&self, renderer: &Self::Type, editable: &CellEditable, path: &str) {
         unsafe {
             let data = T::type_data();
             let parent_class = data.as_ref().parent_class() as *mut ffi::GtkCellRendererClass;
             if let Some(f) = (*parent_class).editing_started {
                 f(
-                    self.obj()
-                        .unsafe_cast_ref::<CellRenderer>()
-                        .to_glib_none()
-                        .0,
+                    renderer.unsafe_cast_ref::<CellRenderer>().to_glib_none().0,
                     editable.to_glib_none().0,
                     path.to_glib_none().0,
                 )
@@ -395,55 +424,107 @@ impl<T: CellRendererImpl> CellRendererImplExt for T {
 
 unsafe impl<T: CellRendererImpl> IsSubclassable<T> for CellRenderer {
     fn class_init(class: &mut ::glib::Class<Self>) {
-        Self::parent_class_init::<T>(class);
+        <Object as IsSubclassable<T>>::class_init(class);
 
         if !crate::rt::is_initialized() {
             panic!("GTK has to be initialized first");
         }
 
         let klass = class.as_mut();
-        klass.get_request_mode = Some(cell_renderer_get_request_mode::<T>);
-        klass.get_preferred_width = Some(cell_renderer_get_preferred_width::<T>);
+
+        klass.activate = Some(cell_renderer_activate::<T>);
+        klass.editing_canceled = Some(cell_renderer_editing_canceled::<T>);
+        klass.editing_started = Some(cell_renderer_editing_started::<T>);
+        klass.get_aligned_area = Some(cell_renderer_get_aligned_area::<T>);
         klass.get_preferred_height_for_width =
             Some(cell_renderer_get_preferred_height_for_width::<T>);
         klass.get_preferred_height = Some(cell_renderer_get_preferred_height::<T>);
         klass.get_preferred_width_for_height =
             Some(cell_renderer_get_preferred_width_for_height::<T>);
-        klass.get_aligned_area = Some(cell_renderer_get_aligned_area::<T>);
-        klass.render = Some(cell_renderer_render::<T>);
-        klass.activate = Some(cell_renderer_activate::<T>);
+        klass.get_preferred_width = Some(cell_renderer_get_preferred_width::<T>);
+        klass.get_request_mode = Some(cell_renderer_get_request_mode::<T>);
+        klass.snapshot = Some(cell_renderer_snapshot::<T>);
         klass.start_editing = Some(cell_renderer_start_editing::<T>);
-        klass.editing_started = Some(cell_renderer_editing_started::<T>);
-        klass.editing_canceled = Some(cell_renderer_editing_canceled::<T>);
+    }
+
+    fn instance_init(instance: &mut glib::subclass::InitializingObject<T>) {
+        <Object as IsSubclassable<T>>::instance_init(instance);
     }
 }
 
-unsafe extern "C" fn cell_renderer_get_request_mode<T: CellRendererImpl>(
+unsafe extern "C" fn cell_renderer_activate<T: CellRendererImpl>(
     ptr: *mut ffi::GtkCellRenderer,
-) -> ffi::GtkSizeRequestMode {
+    evtptr: *mut gdk::ffi::GdkEvent,
+    wdgtptr: *mut ffi::GtkWidget,
+    pathptr: *const c_char,
+    bgptr: *const gdk::ffi::GdkRectangle,
+    cellptr: *const gdk::ffi::GdkRectangle,
+    flags: ffi::GtkCellRendererState,
+) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
+    let imp = instance.impl_();
+    let wrap: Borrowed<CellRenderer> = from_glib_borrow(ptr);
+    let widget: Borrowed<Widget> = from_glib_borrow(wdgtptr);
+    let evt: Borrowed<Option<gdk::Event>> = from_glib_borrow(evtptr);
 
-    imp.request_mode().into_glib()
+    imp.activate(
+        wrap.unsafe_cast_ref(),
+        evt.as_ref().as_ref(),
+        &*widget,
+        &GString::from_glib_borrow(pathptr),
+        &from_glib_borrow(bgptr),
+        &from_glib_borrow(cellptr),
+        from_glib(flags),
+    )
+    .into_glib()
 }
 
-unsafe extern "C" fn cell_renderer_get_preferred_width<T: CellRendererImpl>(
+unsafe extern "C" fn cell_renderer_editing_canceled<T: CellRendererImpl>(
     ptr: *mut ffi::GtkCellRenderer,
-    wdgtptr: *mut ffi::GtkWidget,
-    minptr: *mut c_int,
-    natptr: *mut c_int,
 ) {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
+    let imp = instance.impl_();
+    let wrap: Borrowed<CellRenderer> = from_glib_borrow(ptr);
+
+    imp.editing_canceled(wrap.unsafe_cast_ref());
+}
+
+unsafe extern "C" fn cell_renderer_editing_started<T: CellRendererImpl>(
+    ptr: *mut ffi::GtkCellRenderer,
+    editableptr: *mut ffi::GtkCellEditable,
+    pathptr: *const c_char,
+) {
+    let instance = &*(ptr as *mut T::Instance);
+    let imp = instance.impl_();
+    let wrap: Borrowed<CellRenderer> = from_glib_borrow(ptr);
+    let editable = from_glib_borrow(editableptr);
+
+    imp.editing_started(
+        wrap.unsafe_cast_ref(),
+        &editable,
+        &GString::from_glib_borrow(pathptr),
+    );
+}
+
+unsafe extern "C" fn cell_renderer_get_aligned_area<T: CellRendererImpl>(
+    ptr: *mut ffi::GtkCellRenderer,
+    wdgtptr: *mut ffi::GtkWidget,
+    flags: ffi::GtkCellRendererState,
+    cellarea: *const gdk::ffi::GdkRectangle,
+    alignedptr: *mut gdk::ffi::GdkRectangle,
+) {
+    let instance = &*(ptr as *mut T::Instance);
+    let imp = instance.impl_();
+    let wrap: Borrowed<CellRenderer> = from_glib_borrow(ptr);
     let widget: Borrowed<Widget> = from_glib_borrow(wdgtptr);
 
-    let (min_size, nat_size) = imp.preferred_width(&*widget);
-    if !minptr.is_null() {
-        *minptr = min_size;
-    }
-    if !natptr.is_null() {
-        *natptr = nat_size;
-    }
+    let rectangle = imp.aligned_area(
+        wrap.unsafe_cast_ref(),
+        &*widget,
+        from_glib(flags),
+        &from_glib_borrow(cellarea),
+    );
+    *alignedptr = *rectangle.to_glib_none().0;
 }
 
 unsafe extern "C" fn cell_renderer_get_preferred_height_for_width<T: CellRendererImpl>(
@@ -454,10 +535,12 @@ unsafe extern "C" fn cell_renderer_get_preferred_height_for_width<T: CellRendere
     nat_height_ptr: *mut c_int,
 ) {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
+    let imp = instance.impl_();
+    let wrap: Borrowed<CellRenderer> = from_glib_borrow(ptr);
     let widget: Borrowed<Widget> = from_glib_borrow(wdgtptr);
 
-    let (min_height, nat_height) = imp.preferred_height_for_width(&*widget, width);
+    let (min_height, nat_height) =
+        imp.preferred_height_for_width(wrap.unsafe_cast_ref(), &*widget, width);
     if !min_height_ptr.is_null() {
         *min_height_ptr = min_height;
     }
@@ -473,10 +556,11 @@ unsafe extern "C" fn cell_renderer_get_preferred_height<T: CellRendererImpl>(
     natptr: *mut c_int,
 ) {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
+    let imp = instance.impl_();
+    let wrap: Borrowed<CellRenderer> = from_glib_borrow(ptr);
     let widget: Borrowed<Widget> = from_glib_borrow(wdgtptr);
 
-    let (min_size, nat_size) = imp.preferred_height(&*widget);
+    let (min_size, nat_size) = imp.preferred_height(wrap.unsafe_cast_ref(), &*widget);
     if !minptr.is_null() {
         *minptr = min_size;
     }
@@ -493,10 +577,12 @@ unsafe extern "C" fn cell_renderer_get_preferred_width_for_height<T: CellRendere
     nat_width_ptr: *mut c_int,
 ) {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
+    let imp = instance.impl_();
+    let wrap: Borrowed<CellRenderer> = from_glib_borrow(ptr);
     let widget: Borrowed<Widget> = from_glib_borrow(wdgtptr);
 
-    let (min_width, nat_width) = imp.preferred_width_for_height(&*widget, height);
+    let (min_width, nat_width) =
+        imp.preferred_width_for_height(wrap.unsafe_cast_ref(), &*widget, height);
     if !min_width_ptr.is_null() {
         *min_width_ptr = min_width;
     }
@@ -505,36 +591,53 @@ unsafe extern "C" fn cell_renderer_get_preferred_width_for_height<T: CellRendere
     }
 }
 
-unsafe extern "C" fn cell_renderer_get_aligned_area<T: CellRendererImpl>(
+unsafe extern "C" fn cell_renderer_get_preferred_width<T: CellRendererImpl>(
     ptr: *mut ffi::GtkCellRenderer,
     wdgtptr: *mut ffi::GtkWidget,
-    flags: ffi::GtkCellRendererState,
-    cellarea: *const gdk::ffi::GdkRectangle,
-    alignedptr: *mut gdk::ffi::GdkRectangle,
+    minptr: *mut c_int,
+    natptr: *mut c_int,
 ) {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
+    let imp = instance.impl_();
+    let wrap: Borrowed<CellRenderer> = from_glib_borrow(ptr);
     let widget: Borrowed<Widget> = from_glib_borrow(wdgtptr);
 
-    let rectangle = imp.aligned_area(&*widget, from_glib(flags), &from_glib_borrow(cellarea));
-    *alignedptr = *rectangle.to_glib_none().0;
+    let (min_size, nat_size) = imp.preferred_width(wrap.unsafe_cast_ref(), &*widget);
+    if !minptr.is_null() {
+        *minptr = min_size;
+    }
+    if !natptr.is_null() {
+        *natptr = nat_size;
+    }
 }
 
-unsafe extern "C" fn cell_renderer_render<T: CellRendererImpl>(
+unsafe extern "C" fn cell_renderer_get_request_mode<T: CellRendererImpl>(
     ptr: *mut ffi::GtkCellRenderer,
-    crptr: *mut cairo::ffi::cairo_t,
+) -> ffi::GtkSizeRequestMode {
+    let instance = &*(ptr as *mut T::Instance);
+    let imp = instance.impl_();
+    let wrap: Borrowed<CellRenderer> = from_glib_borrow(ptr);
+
+    imp.request_mode(wrap.unsafe_cast_ref()).into_glib()
+}
+
+unsafe extern "C" fn cell_renderer_snapshot<T: CellRendererImpl>(
+    ptr: *mut ffi::GtkCellRenderer,
+    snapshotptr: *mut ffi::GtkSnapshot,
     wdgtptr: *mut ffi::GtkWidget,
     bgptr: *const gdk::ffi::GdkRectangle,
     cellptr: *const gdk::ffi::GdkRectangle,
     flags: ffi::GtkCellRendererState,
 ) {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
+    let imp = instance.impl_();
+    let wrap: Borrowed<CellRenderer> = from_glib_borrow(ptr);
     let widget: Borrowed<Widget> = from_glib_borrow(wdgtptr);
-    let cr: Borrowed<cairo::Context> = from_glib_borrow(crptr);
+    let snapshot: Borrowed<Snapshot> = from_glib_borrow(snapshotptr);
 
-    imp.render(
-        &cr,
+    imp.snapshot(
+        wrap.unsafe_cast_ref(),
+        &snapshot,
         &*widget,
         &from_glib_borrow(bgptr),
         &from_glib_borrow(cellptr),
@@ -542,31 +645,6 @@ unsafe extern "C" fn cell_renderer_render<T: CellRendererImpl>(
     );
 }
 
-unsafe extern "C" fn cell_renderer_activate<T: CellRendererImpl>(
-    ptr: *mut ffi::GtkCellRenderer,
-    evtptr: *mut gdk::ffi::GdkEvent,
-    wdgtptr: *mut ffi::GtkWidget,
-    pathptr: *const c_char,
-    bgptr: *const gdk::ffi::GdkRectangle,
-    cellptr: *const gdk::ffi::GdkRectangle,
-    flags: ffi::GtkCellRendererState,
-) -> glib::ffi::gboolean {
-    let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
-    let widget: Borrowed<Widget> = from_glib_borrow(wdgtptr);
-    let evt: Borrowed<Option<gdk::Event>> = from_glib_borrow(evtptr);
-
-    imp.activate(
-        evt.as_ref().as_ref(),
-        &*widget,
-        &GString::from_glib_borrow(pathptr),
-        &from_glib_borrow(bgptr),
-        &from_glib_borrow(cellptr),
-        from_glib(flags),
-    )
-    .into_glib()
-}
-
 unsafe extern "C" fn cell_renderer_start_editing<T: CellRendererImpl>(
     ptr: *mut ffi::GtkCellRenderer,
     evtptr: *mut gdk::ffi::GdkEvent,
@@ -577,11 +655,13 @@ unsafe extern "C" fn cell_renderer_start_editing<T: CellRendererImpl>(
     flags: ffi::GtkCellRendererState,
 ) -> *mut ffi::GtkCellEditable {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
+    let imp = instance.impl_();
+    let wrap: Borrowed<CellRenderer> = from_glib_borrow(ptr);
     let widget: Borrowed<Widget> = from_glib_borrow(wdgtptr);
     let evt: Borrowed<Option<gdk::Event>> = from_glib_borrow(evtptr);
 
     imp.start_editing(
+        wrap.unsafe_cast_ref(),
         evt.as_ref().as_ref(),
         &*widget,
         &GString::from_glib_borrow(pathptr),
@@ -592,24 +672,3 @@ unsafe extern "C" fn cell_renderer_start_editing<T: CellRendererImpl>(
     .to_glib_none()
     .0
 }
-
-unsafe extern "C" fn cell_renderer_editing_canceled<T: CellRendererImpl>(
-    ptr: *mut ffi::GtkCellRenderer,
-) {
-    let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
-
-    imp.editing_canceled();
-}
-
-unsafe extern "C" fn cell_renderer_editing_started<T: CellRendererImpl>(
-    ptr: *mut ffi::GtkCellRenderer,
-    editableptr: *mut ffi::GtkCellEditable,
-    pathptr: *const c_char,
-) {
-    let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
-    let editable = from_glib_borrow(editableptr);
-
-    imp.editing_started(&editable, &GString::from_glib_borrow(pathptr));
-}
