@@ -1,33 +1,28 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use libc::c_char;
-
-use glib::subclass::prelude::*;
+use crate::subclass::prelude::*;
+use crate::{CellRenderer, CellRendererText};
 use glib::translate::*;
 use glib::{Cast, GString};
 
-use super::cell_renderer::CellRendererImpl;
-
-use crate::CellRendererText;
-
 pub trait CellRendererTextImpl: CellRendererTextImplExt + CellRendererImpl {
-    fn edited(&self, path: &str, new_text: &str) {
-        self.parent_edited(path, new_text);
+    fn edited(&self, renderer: &Self::Type, path: &str, new_text: &str) {
+        self.parent_edited(renderer, path, new_text);
     }
 }
 
 pub trait CellRendererTextImplExt: ObjectSubclass {
-    fn parent_edited(&self, path: &str, new_text: &str);
+    fn parent_edited(&self, renderer: &Self::Type, path: &str, new_text: &str);
 }
 
 impl<T: CellRendererTextImpl> CellRendererTextImplExt for T {
-    fn parent_edited(&self, path: &str, new_text: &str) {
+    fn parent_edited(&self, renderer: &Self::Type, path: &str, new_text: &str) {
         unsafe {
             let data = T::type_data();
             let parent_class = data.as_ref().parent_class() as *mut ffi::GtkCellRendererTextClass;
             if let Some(f) = (*parent_class).edited {
                 f(
-                    self.obj()
+                    renderer
                         .unsafe_cast_ref::<CellRendererText>()
                         .to_glib_none()
                         .0,
@@ -41,26 +36,28 @@ impl<T: CellRendererTextImpl> CellRendererTextImplExt for T {
 
 unsafe impl<T: CellRendererTextImpl> IsSubclassable<T> for CellRendererText {
     fn class_init(class: &mut ::glib::Class<Self>) {
-        Self::parent_class_init::<T>(class);
-
-        if !crate::rt::is_initialized() {
-            panic!("GTK has to be initialized first");
-        }
+        <CellRenderer as IsSubclassable<T>>::class_init(class);
 
         let klass = class.as_mut();
         klass.edited = Some(cell_renderer_text_edited::<T>);
     }
+
+    fn instance_init(instance: &mut glib::subclass::InitializingObject<T>) {
+        <CellRenderer as IsSubclassable<T>>::instance_init(instance);
+    }
 }
 
 unsafe extern "C" fn cell_renderer_text_edited<T: CellRendererTextImpl>(
     ptr: *mut ffi::GtkCellRendererText,
-    path: *const c_char,
-    new_text: *const c_char,
+    path: *const libc::c_char,
+    new_text: *const libc::c_char,
 ) {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
+    let imp = instance.impl_();
+    let wrap: Borrowed<CellRendererText> = from_glib_borrow(ptr);
 
     imp.edited(
+        wrap.unsafe_cast_ref(),
         &GString::from_glib_borrow(path),
         &GString::from_glib_borrow(new_text),
     )
