@@ -1,30 +1,27 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use glib::subclass::prelude::*;
+use crate::subclass::prelude::*;
+use crate::{Button, ToggleButton};
 use glib::translate::*;
 use glib::Cast;
 
-use super::button::ButtonImpl;
-use crate::ToggleButton;
-
 pub trait ToggleButtonImpl: ToggleButtonImplExt + ButtonImpl {
-    fn toggled(&self) {
-        self.parent_toggled()
+    fn toggled(&self, toggle_button: &Self::Type) {
+        self.parent_toggled(toggle_button)
     }
 }
 
 pub trait ToggleButtonImplExt: ObjectSubclass {
-    fn parent_toggled(&self);
+    fn parent_toggled(&self, toggle_button: &Self::Type);
 }
 
 impl<T: ToggleButtonImpl> ToggleButtonImplExt for T {
-    fn parent_toggled(&self) {
+    fn parent_toggled(&self, toggle_button: &Self::Type) {
         unsafe {
             let data = T::type_data();
             let parent_class = data.as_ref().parent_class() as *mut ffi::GtkToggleButtonClass;
             if let Some(f) = (*parent_class).toggled {
-                f(self
-                    .obj()
+                f(toggle_button
                     .unsafe_cast_ref::<ToggleButton>()
                     .to_glib_none()
                     .0)
@@ -35,16 +32,21 @@ impl<T: ToggleButtonImpl> ToggleButtonImplExt for T {
 
 unsafe impl<T: ToggleButtonImpl> IsSubclassable<T> for ToggleButton {
     fn class_init(class: &mut glib::Class<Self>) {
-        Self::parent_class_init::<T>(class);
+        <Button as IsSubclassable<T>>::class_init(class);
 
         let klass = class.as_mut();
-        klass.toggled = Some(button_toggled::<T>);
+        klass.toggled = Some(toggle_button_toggled::<T>);
+    }
+
+    fn instance_init(instance: &mut glib::subclass::InitializingObject<T>) {
+        <Button as IsSubclassable<T>>::instance_init(instance);
     }
 }
 
-unsafe extern "C" fn button_toggled<T: ToggleButtonImpl>(ptr: *mut ffi::GtkToggleButton) {
+unsafe extern "C" fn toggle_button_toggled<T: ToggleButtonImpl>(ptr: *mut ffi::GtkToggleButton) {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.imp();
+    let imp = instance.impl_();
+    let wrap: Borrowed<ToggleButton> = from_glib_borrow(ptr);
 
-    imp.toggled()
+    imp.toggled(wrap.unsafe_cast_ref())
 }
