@@ -1,15 +1,12 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use crate::SortType;
-use glib::object::IsA;
+use glib::object::{Cast, IsA};
 use glib::translate::*;
 use std::cmp::Ordering;
 use std::fmt;
 use std::mem;
 
-use crate::{TreeIter, TreeModel, TreeSortable};
-use ffi::{GtkTreeIter, GtkTreeModel};
-use glib::ffi::gpointer;
+use crate::{SortType, TreeIter, TreeModel, TreeSortable};
 
 #[derive(Clone, Copy, Debug, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub enum SortColumn {
@@ -25,8 +22,8 @@ impl IntoGlib for SortColumn {
     #[inline]
     fn into_glib(self) -> i32 {
         match self {
-            Self::Default => ffi::GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
-            Self::Index(x) => {
+            SortColumn::Default => ffi::GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
+            SortColumn::Index(x) => {
                 assert!(x <= i32::max_value() as u32, "column index is too big");
                 x as i32
             }
@@ -55,8 +52,8 @@ impl fmt::Display for SortColumn {
             f,
             "SortColumn::{}",
             match *self {
-                Self::Default => "Default",
-                Self::Index(_) => "Index",
+                SortColumn::Default => "Default",
+                SortColumn::Index(_) => "Index",
             }
         )
     }
@@ -66,26 +63,51 @@ pub trait TreeSortableExtManual: 'static {
     #[doc(alias = "gtk_tree_sortable_set_default_sort_func")]
     fn set_default_sort_func<F>(&self, sort_func: F)
     where
-        F: Fn(&TreeModel, &TreeIter, &TreeIter) -> Ordering + 'static;
+        F: Fn(&Self, &TreeIter, &TreeIter) -> Ordering + 'static;
     #[doc(alias = "gtk_tree_sortable_set_sort_func")]
     fn set_sort_func<F>(&self, sort_column_id: SortColumn, sort_func: F)
     where
-        F: Fn(&TreeModel, &TreeIter, &TreeIter) -> Ordering + 'static;
-    #[doc(alias = "get_sort_column_id")]
+        F: Fn(&Self, &TreeIter, &TreeIter) -> Ordering + 'static;
     #[doc(alias = "gtk_tree_sortable_get_sort_column_id")]
+    #[doc(alias = "get_sort_column_id")]
     fn sort_column_id(&self) -> Option<(SortColumn, SortType)>;
     #[doc(alias = "gtk_tree_sortable_set_sort_column_id")]
     fn set_sort_column_id(&self, sort_column_id: SortColumn, order: SortType);
+    #[doc(alias = "gtk_tree_sortable_set_sort_column_id")]
     fn set_unsorted(&self);
 }
 
-fn into_raw<F, T>(func: F) -> gpointer
+unsafe extern "C" fn trampoline<T, F: Fn(&T, &TreeIter, &TreeIter) -> Ordering>(
+    this: *mut ffi::GtkTreeModel,
+    iter: *mut ffi::GtkTreeIter,
+    iter2: *mut ffi::GtkTreeIter,
+    f: glib::ffi::gpointer,
+) -> i32
+where
+    T: IsA<TreeSortable>,
+{
+    let f: &F = &*(f as *const F);
+    f(
+        &TreeModel::from_glib_none(this).unsafe_cast(),
+        &from_glib_borrow(iter),
+        &from_glib_borrow(iter2),
+    )
+    .into_glib()
+}
+
+unsafe extern "C" fn destroy_closure<T, F: Fn(&T, &TreeIter, &TreeIter) -> Ordering>(
+    ptr: glib::ffi::gpointer,
+) {
+    Box::<F>::from_raw(ptr as *mut _);
+}
+
+fn into_raw<F, T>(func: F) -> glib::ffi::gpointer
 where
     F: Fn(&T, &TreeIter, &TreeIter) -> Ordering + 'static,
 {
     skip_assert_initialized!();
     let func: Box<F> = Box::new(func);
-    Box::into_raw(func) as gpointer
+    Box::into_raw(func) as glib::ffi::gpointer
 }
 
 impl<O: IsA<TreeSortable>> TreeSortableExtManual for O {
@@ -110,35 +132,14 @@ impl<O: IsA<TreeSortable>> TreeSortableExtManual for O {
 
     fn set_default_sort_func<F>(&self, sort_func: F)
     where
-        F: Fn(&TreeModel, &TreeIter, &TreeIter) -> Ordering + 'static,
+        F: Fn(&Self, &TreeIter, &TreeIter) -> Ordering + 'static,
     {
-        unsafe extern "C" fn trampoline<F: Fn(&TreeModel, &TreeIter, &TreeIter) -> Ordering>(
-            this: *mut GtkTreeModel,
-            iter: *mut GtkTreeIter,
-            iter2: *mut GtkTreeIter,
-            f: gpointer,
-        ) -> i32 {
-            let f: &F = &*(f as *const F);
-            f(
-                &TreeModel::from_glib_borrow(this),
-                &from_glib_borrow(iter),
-                &from_glib_borrow(iter2),
-            )
-            .into_glib()
-        }
-        unsafe extern "C" fn destroy_closure<
-            F: Fn(&TreeModel, &TreeIter, &TreeIter) -> Ordering,
-        >(
-            ptr: gpointer,
-        ) {
-            let _ = Box::<F>::from_raw(ptr as *mut _);
-        }
         unsafe {
             ffi::gtk_tree_sortable_set_default_sort_func(
                 self.as_ref().to_glib_none().0,
-                Some(trampoline::<F>),
+                Some(trampoline::<Self, F>),
                 into_raw(sort_func),
-                Some(destroy_closure::<F>),
+                Some(destroy_closure::<Self, F>),
             )
         }
     }
@@ -166,36 +167,15 @@ impl<O: IsA<TreeSortable>> TreeSortableExtManual for O {
 
     fn set_sort_func<F>(&self, sort_column_id: SortColumn, sort_func: F)
     where
-        F: Fn(&TreeModel, &TreeIter, &TreeIter) -> Ordering + 'static,
+        F: Fn(&Self, &TreeIter, &TreeIter) -> Ordering + 'static,
     {
-        unsafe extern "C" fn trampoline<F: Fn(&TreeModel, &TreeIter, &TreeIter) -> Ordering>(
-            this: *mut GtkTreeModel,
-            iter: *mut GtkTreeIter,
-            iter2: *mut GtkTreeIter,
-            f: gpointer,
-        ) -> i32 {
-            let f: &F = &*(f as *const F);
-            f(
-                &TreeModel::from_glib_borrow(this),
-                &from_glib_borrow(iter),
-                &from_glib_borrow(iter2),
-            )
-            .into_glib()
-        }
-        unsafe extern "C" fn destroy_closure<
-            F: Fn(&TreeModel, &TreeIter, &TreeIter) -> Ordering,
-        >(
-            ptr: gpointer,
-        ) {
-            let _ = Box::<F>::from_raw(ptr as *mut _);
-        }
         unsafe {
             ffi::gtk_tree_sortable_set_sort_func(
                 self.as_ref().to_glib_none().0,
                 sort_column_id.into_glib(),
-                Some(trampoline::<F>),
+                Some(trampoline::<Self, F>),
                 into_raw(sort_func),
-                Some(destroy_closure::<F>),
+                Some(destroy_closure::<Self, F>),
             )
         }
     }
