@@ -1,16 +1,14 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use crate::TreeIter;
-use crate::TreeModel;
-use crate::TreeStore;
-use glib::object::{Cast, IsA};
+use crate::{TreeIter, TreeModel, TreeStore};
 use glib::translate::*;
-use glib::{ToValue, Type, Value};
+use glib::{Cast, ToValue, Type, Value};
 use libc::c_int;
 
 impl TreeStore {
     #[doc(alias = "gtk_tree_store_newv")]
-    pub fn new(column_types: &[Type]) -> TreeStore {
+    #[doc(alias = "gtk_tree_store_new")]
+    pub fn new(column_types: &[Type]) -> Self {
         assert_initialized_main_thread!();
         unsafe {
             let mut column_types = column_types
@@ -23,30 +21,10 @@ impl TreeStore {
             ))
         }
     }
-}
 
-pub trait TreeStoreExtManual: 'static {
+    #[doc(alias = "gtk_tree_store_insert_with_values")]
     #[doc(alias = "gtk_tree_store_insert_with_valuesv")]
-    fn insert_with_values(
-        &self,
-        parent: Option<&TreeIter>,
-        position: Option<u32>,
-        columns_and_values: &[(u32, &dyn ToValue)],
-    ) -> TreeIter;
-
-    #[doc(alias = "gtk_tree_store_reorder")]
-    fn reorder(&self, parent: &TreeIter, new_order: &[u32]);
-
-    #[doc(alias = "gtk_tree_store_set")]
-    #[doc(alias = "gtk_tree_store_set_valuesv")]
-    fn set(&self, iter: &TreeIter, columns_and_values: &[(u32, &dyn ToValue)]);
-
-    #[doc(alias = "gtk_tree_store_set_value")]
-    fn set_value(&self, iter: &TreeIter, column: u32, value: &Value);
-}
-
-impl<O: IsA<TreeStore>> TreeStoreExtManual for O {
-    fn insert_with_values(
+    pub fn insert_with_values(
         &self,
         parent: Option<&TreeIter>,
         position: Option<u32>,
@@ -58,28 +36,31 @@ impl<O: IsA<TreeStore>> TreeStoreExtManual for O {
                 "can't have more than {} rows",
                 i32::max_value()
             );
-            let n_columns = ffi::gtk_tree_model_get_n_columns(
-                self.as_ref().upcast_ref::<TreeModel>().to_glib_none().0,
-            ) as u32;
+            let n_columns =
+                ffi::gtk_tree_model_get_n_columns(self.upcast_ref::<TreeModel>().to_glib_none().0)
+                    as u32;
             assert!(
                 columns_and_values.len() <= n_columns as usize,
-                "got values for {} columns but only {n_columns} columns exist",
+                "got values for {} columns but only {} columns exist",
                 columns_and_values.len(),
+                n_columns
             );
             for (column, value) in columns_and_values {
                 assert!(
                     *column < n_columns,
-                    "got column {} which is higher than the number of columns {n_columns}",
+                    "got column {} which is higher than the number of columns {}",
                     *column,
+                    n_columns
                 );
                 let type_ = from_glib(ffi::gtk_tree_model_get_column_type(
-                    self.as_ref().upcast_ref::<TreeModel>().to_glib_none().0,
+                    self.upcast_ref::<TreeModel>().to_glib_none().0,
                     *column as c_int,
                 ));
                 assert!(
                     Value::type_transformable(value.value_type(), type_),
-                    "column {} is of type {type_} but found value of type {}",
+                    "column {} is of type {} but found value of type {}",
                     *column,
+                    type_,
                     value.value_type()
                 );
             }
@@ -95,7 +76,7 @@ impl<O: IsA<TreeStore>> TreeStoreExtManual for O {
 
             let mut iter = TreeIter::uninitialized();
             ffi::gtk_tree_store_insert_with_valuesv(
-                self.as_ref().to_glib_none().0,
+                self.to_glib_none().0,
                 iter.to_glib_none_mut().0,
                 mut_override(parent.to_glib_none().0),
                 position.map_or(-1, |n| n as c_int),
@@ -107,16 +88,18 @@ impl<O: IsA<TreeStore>> TreeStoreExtManual for O {
         }
     }
 
-    fn reorder(&self, parent: &TreeIter, new_order: &[u32]) {
+    #[doc(alias = "gtk_tree_store_reorder")]
+    pub fn reorder(&self, parent: &TreeIter, new_order: &[u32]) {
         unsafe {
             let count = ffi::gtk_tree_model_iter_n_children(
-                self.as_ref().upcast_ref::<TreeModel>().to_glib_none().0,
+                self.upcast_ref::<TreeModel>().to_glib_none().0,
                 mut_override(parent.to_glib_none().0),
             );
             let safe_count = count as usize == new_order.len();
             debug_assert!(
                 safe_count,
-                "Incorrect `new_order` slice length. Expected `{count}`, found `{}`.",
+                "Incorrect `new_order` slice length. Expected `{}`, found `{}`.",
+                count,
                 new_order.len()
             );
             let safe_values = new_order.iter().max().map_or(true, |&max| {
@@ -126,12 +109,13 @@ impl<O: IsA<TreeStore>> TreeStoreExtManual for O {
             debug_assert!(
                 safe_values,
                 "Some `new_order` slice values are out of range. Maximum safe value: \
-                 `{}`. The slice contents: `{new_order:?}`",
+                 `{}`. The slice contents: `{:?}`",
                 count - 1,
+                new_order
             );
             if safe_count && safe_values {
                 ffi::gtk_tree_store_reorder(
-                    self.as_ref().to_glib_none().0,
+                    self.to_glib_none().0,
                     mut_override(parent.to_glib_none().0),
                     mut_override(new_order.as_ptr() as *const c_int),
                 );
@@ -139,11 +123,14 @@ impl<O: IsA<TreeStore>> TreeStoreExtManual for O {
         }
     }
 
-    fn set(&self, iter: &TreeIter, columns_and_values: &[(u32, &dyn ToValue)]) {
+    #[doc(alias = "gtk_tree_store_set")]
+    #[doc(alias = "gtk_tree_store_set_valuesv")]
+    #[doc(alias = "gtk_tree_store_set_valist")]
+    pub fn set(&self, iter: &TreeIter, columns_and_values: &[(u32, &dyn ToValue)]) {
         unsafe {
-            let n_columns = ffi::gtk_tree_model_get_n_columns(
-                self.as_ref().upcast_ref::<TreeModel>().to_glib_none().0,
-            ) as u32;
+            let n_columns =
+                ffi::gtk_tree_model_get_n_columns(self.upcast_ref::<TreeModel>().to_glib_none().0)
+                    as u32;
             assert!(
                 columns_and_values.len() <= n_columns as usize,
                 "got values for {} columns but only {} columns exist",
@@ -153,17 +140,19 @@ impl<O: IsA<TreeStore>> TreeStoreExtManual for O {
             for (column, value) in columns_and_values {
                 assert!(
                     *column < n_columns,
-                    "got column {} which is higher than the number of columns {n_columns}",
+                    "got column {} which is higher than the number of columns {}",
                     *column,
+                    n_columns
                 );
                 let type_ = from_glib(ffi::gtk_tree_model_get_column_type(
-                    self.as_ref().upcast_ref::<TreeModel>().to_glib_none().0,
+                    self.upcast_ref::<TreeModel>().to_glib_none().0,
                     *column as c_int,
                 ));
                 assert!(
                     Value::type_transformable(value.value_type(), type_),
-                    "column {} is of type {type_} but found value of type {}",
+                    "column {} is of type {} but found value of type {}",
                     *column,
+                    type_,
                     value.value_type()
                 );
             }
@@ -178,7 +167,7 @@ impl<O: IsA<TreeStore>> TreeStoreExtManual for O {
                 .collect::<Vec<_>>();
 
             ffi::gtk_tree_store_set_valuesv(
-                self.as_ref().to_glib_none().0,
+                self.to_glib_none().0,
                 mut_override(iter.to_glib_none().0),
                 mut_override(columns.as_ptr() as *const c_int),
                 mut_override(values.as_ptr() as *const glib::gobject_ffi::GValue),
@@ -187,28 +176,45 @@ impl<O: IsA<TreeStore>> TreeStoreExtManual for O {
         }
     }
 
-    fn set_value(&self, iter: &TreeIter, column: u32, value: &Value) {
+    #[doc(alias = "gtk_tree_store_set_column_types")]
+    pub fn set_column_types(&self, types: &[glib::Type]) {
         unsafe {
-            let columns = ffi::gtk_tree_model_get_n_columns(
-                self.as_ref().upcast_ref::<TreeModel>().to_glib_none().0,
-            ) as u32;
+            let types_ptr: Vec<glib::ffi::GType> = types.iter().map(|t| t.into_glib()).collect();
+            ffi::gtk_tree_store_set_column_types(
+                self.to_glib_none().0,
+                types.len() as i32,
+                mut_override(types_ptr.as_ptr()),
+            )
+        }
+    }
+
+    #[doc(alias = "gtk_tree_store_set_value")]
+    pub fn set_value(&self, iter: &TreeIter, column: u32, value: &Value) {
+        unsafe {
+            let columns =
+                ffi::gtk_tree_model_get_n_columns(self.upcast_ref::<TreeModel>().to_glib_none().0)
+                    as u32;
             assert!(
                 column < columns,
-                "got column {column} which is higher than the number of columns {columns}",
+                "got column {} which is higher than the number of columns {}",
+                column,
+                columns
             );
 
             let type_ = from_glib(ffi::gtk_tree_model_get_column_type(
-                self.as_ref().upcast_ref::<TreeModel>().to_glib_none().0,
+                self.upcast_ref::<TreeModel>().to_glib_none().0,
                 column as c_int,
             ));
             assert!(
                 Value::type_transformable(value.type_(), type_),
-                "column {column} is of type {type_} but found value of type {}",
+                "column {} is of type {} but found value of type {}",
+                column,
+                type_,
                 value.type_()
             );
 
             ffi::gtk_tree_store_set_value(
-                self.as_ref().to_glib_none().0,
+                self.to_glib_none().0,
                 mut_override(iter.to_glib_none().0),
                 column as c_int,
                 mut_override(value.to_glib_none().0),
